package com.ctpodp;

import android.os.Bundle;
import android.os.Handler;
import android.util.Log;
import android.view.View;
import android.widget.FrameLayout;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import com.facebook.react.bridge.ReactApplicationContext;
import com.facebook.react.common.MapBuilder;
import com.facebook.react.modules.core.DeviceEventManagerModule;
import com.facebook.react.uimanager.SimpleViewManager;
import com.facebook.react.uimanager.ThemedReactContext;
import com.facebook.react.uimanager.events.RCTEventEmitter;
import com.facebook.react.uimanager.events.RCTModernEventEmitter;
import com.facebook.react.views.image.ReactImageView;

import com.facebook.react.bridge.Arguments;
import com.facebook.react.bridge.ReactApplicationContext;
import com.facebook.react.bridge.ReactContext;
import com.facebook.react.bridge.ReactContextBaseJavaModule;
import com.facebook.react.bridge.ReactMethod;
import com.facebook.react.bridge.ReadableArray;
import com.facebook.react.bridge.WritableArray;
import com.facebook.react.bridge.WritableMap;
import com.inpixon.odp.Extend;
import com.inpixon.odp.TDNavOutput;
import com.inpixon.odp.aiding.FloorPlan;
import com.inpixon.odp.aiding.Location;
import com.inpixon.odp.aiding.Region;

import java.util.List;
import java.util.Map;

public class OdpViewManager extends SimpleViewManager<FrameLayout> {

  public static final String REACT_CLASS = "OdpViewManager";
  public final int COMMAND_START = 1;
  public final int COMMAND_UPLOAD_LOGS = 2;
  ReactApplicationContext _reactContext;
  Extend _extend;

  public OdpViewManager(ReactApplicationContext reactContext) {
    _reactContext = reactContext;
  }

  @Override
  public String getName() {
    return REACT_CLASS;
  }

  @NonNull
  @Override
  protected FrameLayout createViewInstance(@NonNull ThemedReactContext themedReactContext) {
    return new FrameLayout(themedReactContext);
  }

  /**
   * Map the "create" command to an integer
   */
  @Nullable
  @Override
  public Map<String, Integer> getCommandsMap() {
    return MapBuilder.of("start", COMMAND_START, "upload_logs", COMMAND_UPLOAD_LOGS);
  }

  /**
   * Handle "create" command (called from JS) and call createFragment method
   */
  @Override
  public void receiveCommand(
    @NonNull FrameLayout root,
    String commandId,
    @Nullable ReadableArray args
  ) {
    super.receiveCommand(root, commandId, args);
    int commandIdInt = Integer.parseInt(commandId);
    String key = args.getString(0);
    String host = args.getString(1);

    switch (commandIdInt) {
      case COMMAND_START:
        start(key, host);
        break;
      case COMMAND_UPLOAD_LOGS:
        uploadLogs();
        break;
      default: {}
    }
  }

  public void sendEvent(String topic, WritableMap data) {
    _reactContext.getJSModule(DeviceEventManagerModule.RCTDeviceEventEmitter.class)
      .emit(topic, data);
  }

  public void start(String key, String host) {
      Log.d("Extend", "ODP Init");
      Toast.makeText(_reactContext, "Extend Initializing", Toast.LENGTH_SHORT).show();
      _extend = new Extend(_reactContext, key, host);
      _extend.setOnExtend(new ExtendListener());
      _extend.bind();
  }

  public void onDestroy() {
    _extend.unbind();
  }

  public void uploadLogs() {
    _extend.uploadLogs();
  }

  class ExtendListener implements Extend.OnExtend {

    @Override
    public void onReady(boolean b) {
      WritableMap params = Arguments.createMap();
      params.putBoolean("boolean", b);
      Log.d("Extend", "Extend Ready");

      sendEvent("onReady", params);
    }

    @Override
    public void onKeyValidation(boolean validated) {
      if (validated) { // The key has been validated. Set whichever modules you want enabled
        // or disabled here and start the service after these calls.
        // Toast.makeText(_reactContext, "Key Valid", Toast.LENGTH_SHORT).show();
        Log.d("Extend", "Extend Started");
        _extend.setLogWriting(true); // enable sensor logging for debug
        _extend.setVicinityEnabled(true);
        _extend.start();
      } else { // The key is invalid or expired.
        // Toast.makeText(_reactContext, "Key Not Valid", Toast.LENGTH_SHORT).show();
        Log.d("Extend", "Extend Not Started");
      }
    }

    @Override
    public void onNavigationUpdate(TDNavOutput navOutput) {
      Log.d("Extend", "onNavigationUpdate: " + navOutput.latitude + ", " + navOutput.longitude);
      // Toast.makeText(_reactContext, navOutput.latitude + ", " +
      // navOutput.longitude, Toast.LENGTH_SHORT).show();
      WritableMap params = Arguments.createMap();
      params.putDouble("lat", navOutput.latitude);
      params.putDouble("long", navOutput.longitude);
      params.putDouble("x", navOutput.x);
      params.putDouble("y", navOutput.y);
      params.putDouble("xfom", navOutput.xfom);
      params.putDouble("yfom", navOutput.yfom);
      params.putDouble("viewHeading", navOutput.viewHeading);
      params.putDouble("bearing", navOutput.bearing);
      params.putDouble("accuracyRadius", navOutput.accuracyRadius);
      params.putInt("floor", navOutput.floor);
      params.putDouble("utc", navOutput.utc);
      params.putInt("dynamicState", navOutput.dynamicState);
      params.putInt("elevationState", navOutput.elevationState);
      params.putInt("usedSystems", navOutput.usedSystems);
      sendEvent("onNavigationUpdate", params);
    }

    @Override
    public void onFacilityUpdate(String name, Location location, Region region, List<FloorPlan> floorPlans) {
      Log.d("Extend", "onFacilityUpdate: " + name);
      WritableMap params = Arguments.createMap();

      // Name
      params.putString("name", name);

      // Location
      WritableMap locationMap = Arguments.createMap();
      locationMap.putDouble("latitude", location.getLatitude());
      locationMap.putDouble("longitude", location.getLongitude());
      locationMap.putDouble("altitude", location.getAltitude());
      locationMap.putDouble("x", location.getX());
      locationMap.putDouble("y", location.getY());
      locationMap.putDouble("z", location.getZ());
      locationMap.putBoolean("isGeo", location.getIsGeo());
      params.putMap("location", locationMap);

      // Region
      WritableMap regionMap = Arguments.createMap();
      locationMap.putDouble("floor", region.getFloor());
      locationMap.putDouble("id", region.getId());
      locationMap.putString("name", region.getName());
      locationMap.putString("icon", region.getIcon());
      WritableArray pointsArray = Arguments.createArray();
      for (Location pointLocation : region.getPoints()) {
        WritableMap pLMap = Arguments.createMap();
        pLMap.putDouble("latitude", pointLocation.getLatitude());
        pLMap.putDouble("longitude", pointLocation.getLongitude());
        pLMap.putDouble("altitude", pointLocation.getAltitude());
        pLMap.putDouble("x", pointLocation.getX());
        pLMap.putDouble("y", pointLocation.getY());
        pLMap.putDouble("x", pointLocation.getZ());
        pLMap.putBoolean("isGeo", pointLocation.getIsGeo());
        pointsArray.pushMap(pLMap);
      }
      locationMap.putArray("points", pointsArray);
      locationMap.putString("className", region.getClass().getSimpleName());
      params.putMap("region", regionMap);

      // FloorPlans
      WritableArray floorPlanArr = Arguments.createArray();
      for (FloorPlan floorPlan : floorPlans) {
        WritableMap flPlanMap = Arguments.createMap();
        flPlanMap.putDouble("floor", floorPlan.getFloor());
        flPlanMap.putInt("describeContents", floorPlan.describeContents());
        flPlanMap.putDouble("latitude", floorPlan.getGeoHeight());
        flPlanMap.putDouble("xMax", floorPlan.getXMax());
        flPlanMap.putDouble("xMin", floorPlan.getXMin());
        flPlanMap.putDouble("yMax", floorPlan.getYMax());
        flPlanMap.putDouble("yMin", floorPlan.getYMin());
        flPlanMap.putString("file", floorPlan.getFile());
        flPlanMap.putString("name", floorPlan.getName());
        flPlanMap.putString("className", floorPlan.getClass().getSimpleName());
        floorPlanArr.pushMap(flPlanMap);
      }
      params.putArray("floorPlans", floorPlanArr);

      sendEvent("onFacilityUpdate", params);
    }

    @Override
    public void onDebugEvent(Bundle bundle) {

    }

    @Override
    public void onUploadProgressUpdate(int i, int i1, int i2, int i3) {

    }
  }


}
